-- trim - Removes leading or trailing whitespace
function trim ( s )
	return string.gsub( s, "^%s*(.-)%s*$", "%1" )
end

-- stripQuotes - Used to strip quotes from strings
function stripQuotes( s )
	return string.gsub( s, "^[\"]*(.-)[\"]*$", "%1" )
end


-- splitArgs - Used for splitting console arguments, will obey quotation marks
function splitArgs( args )
	local argv = {}
	local argc = 0
	local quoteFlag = false
	local strStore = ""
	
	for a in string.gfind( args, "[%a%p%d]+") do
		if a ~= nil then
			if quoteFlag == false then
				if string.sub( a, 1, 1 ) == "\"" and string.sub( a, -1, -1 ) ~= "\"" then -- If we're beginning a quote
					quoteFlag = true
					strStore = a -- Since this is the beginning, assign the store the beginning
				else
					table.insert( argv, a )
					argc = argc + 1
				end
			else -- If quoteFlag is true
				if string.sub( a, -1, -1 ) == "\"" then -- If we're ending our quote
					quoteFlag = false
					strStore = strStore .. " " .. a
					strStore = stripQuotes( strStore )
					table.insert( argv, strStore )
					argc = argc + 1
				else
					strStore = strStore .. " " .. a -- Add to our store
				end
			end
		end
	end
	
	return argv, argc
end

-- print_functioncall - Pass this the name of the function in a string and 
-- all the arguments of a function to get a helpful debug output of the function call
-- EXAMPLE OUTPUT:
-- [ULX] Debug function call: sendTextAndAnimate( (number)1, (number)10, 
-- (number)0, (number)0.3, (number)0.05, (number)0.51, (number)255, (number)255, 
-- (number)255, (string)7. Exit, (string)ChatFont, (number)60, (number)0.5, (number)0.5, 
-- (number)0.5, (number)0.3,  )
function print_functioncall( function_name, ... )
	_Msg( "[ULX] Debug function call: " .. function_name .. "( " )
	for i=1, table.getn( arg ) do
		_Msg( "(" .. type( arg[ i ] ) .. ")" .. tostring( arg[ i ] ) .. ", " )
	end
	_Msg( " )\n" )
end

function debug( text )
	--if DEBUG_MODE then
		_Msg( "[ULX] Debug: " .. text .. "\n" )
	--end
end

function error_msg( text )
	_Msg( "[ULX] ERROR: " .. text .. "\n" )
end

-- sendText - sends the text in the specified format ( used for menus )
function sendText( userid, key, posx, posy, r, g, b, text, font, timeout, fadein, fadeout )
	_GModText_Start( font )
		_GModText_SetPos( posx, posy )
		_GModText_SetColor( r, g, b, 255 )
		_GModText_SetTime( timeout, fadein, fadeout )
		_GModText_SetText( text )
		_GModText_SetDelay(0)
		_GModText_AllowOffscreen( false )
	_GModText_Send( userid, key )
end

-- sendTextAnimate - animates the text ( you have to define the text with sendText or sendTextAndAnimate first! )
function sendTextAnimate( userid, key, posx, posy, r, g, b, font, length, ease )
	_GModText_Start( font )
		_GModText_SetPos( posx, posy )
		_GModText_SetColor( r, g, b, 255 )
		_GModText_AllowOffscreen( false )
	_GModText_SendAnimate( userid, key, length, ease )
end

-- sendTextAndAnimate - sends and animates the text in the specified format ( used for menus )
function sendTextAndAnimate( userid, key, begPosx, begPosy, posx, posy, r, g, b, text, font, timeout, fadein, fadeout, length, ease )
	_GModText_Start( font )
		_GModText_SetPos( begPosx, begPosy )
		_GModText_SetColor( r, g, b, 255 )
		_GModText_SetTime( timeout, fadein, fadeout )
		_GModText_SetText( text )
		_GModText_SetDelay( 0 )
		_GModText_AllowOffscreen( false )
	_GModText_Send( userid, key )
	
		
	_GModText_Start( font );
		_GModText_SetPos( posx, posy )
		_GModText_SetColor( r, g, b, 255 )
		_GModText_AllowOffscreen( false )
	_GModText_SendAnimate( userid, key, length, ease );
end

-- sendRect - creates a rectangle in the specified format ( used for menus )
function sendRect( userid, key, rectBegX, rectBegY1, rectBegSizeX, rectBegSizeY, r, g, b, a, material, timeout, fadein, fadeout )
	_GModRect_Start( material )
		_GModRect_SetPos( rectBegX, rectBegY1, rectBegSizeX, rectBegSizeY )
		_GModRect_SetColor( r, g, b, a )
		_GModRect_SetTime( timeout, fadein, fadeout )
		_GModRect_SetDelay( 0 )
	_GModRect_Send( userid, key )
end

-- sendRectAnimate - animates the rectangle ( you have to define the text with sendRect or sendRectAndAnimate first! )
function sendRectAnimate( userid, key, rectEndX, rectEndY, rectEndSizeX, rectEndSizeY, r, g, b, a, material, length, ease )
	_GModRect_Start( material );
		_GModRect_SetPos( rectEndX, rectEndY, rectEndSizeX, rectEndSizeY )
		_GModRect_SetColor( r, g, b, a )
	_GModRect_SendAnimate( userid, key, length, ease )
end

-- sendRectAndAnimate - sends and animates the rectangle in the specified format ( used for menus )
function sendRectAndAnimate( userid, key, rectBegX, rectBegY1, rectBegSizeX, rectBegSizeY, rectEndX, rectEndY, rectEndSizeX, rectEndSizeY, r, g, b, a, material, timeout, fadein, fadeout, length, ease )
	_GModRect_Start( material )
		_GModRect_SetPos( rectBegX, rectBegY1, rectBegSizeX, rectBegSizeY )
		_GModRect_SetColor( r, g, b, a )
		_GModRect_SetTime( timeout, fadein, fadeout )
		_GModRect_SetDelay( 0 )
	_GModRect_Send( userid, key )
	
	_GModRect_Start( material );
		_GModRect_SetPos( rectEndX, rectEndY, rectEndSizeX, rectEndSizeY )
		_GModRect_SetColor( r, g, b, a )
	_GModRect_SendAnimate( userid, key, length, ease )
end

function ulx_play3DSound( sound, vector, volume, pitch )
	volume = volume or 1
	pitch = pitch or 1
	
	local newent = _EntCreate( "info_null" )
	_EntSetPos( newent, vector )
	_EntSpawn( newent )
	
	_EntEmitSoundEx( newent, sound, volume, pitch )	
end

-- These are the sounds that will be used for slaps
gSlapSounds = {
"physics/body/body_medium_impact_hard1.wav",
"physics/body/body_medium_impact_hard2.wav",
"physics/body/body_medium_impact_hard3.wav",
"physics/body/body_medium_impact_hard5.wav",
"physics/body/body_medium_impact_hard6.wav",
"physics/body/body_medium_impact_soft5.wav",
"physics/body/body_medium_impact_soft6.wav",
"physics/body/body_medium_impact_soft7.wav"
}

function ulx_slap( target, damage, power )	
	-- Play a slap sound
	local sound_num = math.random( table.getn( gSlapSounds ) ) -- Choose at random
	ulx_play3DSound( gSlapSounds[ sound_num ], _EntGetPos( target ), 1.5 )
	
	local vec = { x=math.random( 20 )-10, y=math.random( 20 )-10, z=math.random( 20 )-8 }
	vec = vecNormalize( vec )  -- This should make the slaps more equal in force, no matter what the direction
	vec = vecMul( vec, vector3( power, power, power ) ) -- Now make the slap stronger
	
	_EntSetVelocity( target, vec )
	
	local newHp = _PlayerInfo( target, "health" ) - damage
		
	if newHp <= 0 then
		_PlayerKill( target )
		return
	end
		
	_PlayerSetHealth( target, newHp )
end

-- This will log the message to the console as well as show it in the talk area
function ulx_tsay( userid, msg )
	_PrintMessage( userid, HUD_PRINTTALK, msg .. "\n" )
	_PrintMessage( userid, HUD_PRINTCONSOLE, msg .. "\n" )	
end

-- This will log the message to the console as well as show it in the talk area
function ulx_tsay_all( msg )
	_PrintMessageAll( HUD_PRINTTALK, msg .. "\n" )
	_PrintMessageAll( HUD_PRINTCONSOLE, msg .. "\n" )	
end

-- This will log the message to the console as well as show it in the center
function ulx_csay( userid, msg, r, g, b, length )
	r = r or 255
	g = g or 255
	b = b or 255
	length = length or 5
	sendText( userid, newTextKey(), 0.5 - string.len( msg )/200, 0.35, r, g, b, msg .. "\n", "TargetID", length, 0.5, 0.5 )
	_PrintMessage( userid, HUD_PRINTCONSOLE, msg .. "\n" )	
end

-- This will log the message to the console as well as show it in the center
function ulx_csay_all( msg, r, g, b )
	r = r or 255
	g = g or 255
	b = b or 255
	length = length or 5
	sendText( 0, newTextKey(), 0.5 - string.len( msg )/200, 0.35, r, g, b, msg .. "\n", "TargetID", length, 0.5, 0.5 )
	_PrintMessageAll( HUD_PRINTCONSOLE, msg .. "\n" )	
end

-- This will log the message to the console
function ulx_console( userid, msg )
	_PrintMessage( userid, HUD_PRINTCONSOLE, msg .. "\n" )	
end

-- This will log the message to the console
function ulx_console_all( msg )
	_PrintMessageAll( HUD_PRINTCONSOLE, msg .. "\n" )	
end

-- Thanks goes to andyvincent for inspiring getFilesFromDir
function getFilesFromDir( dir )
	if _file.Exists( dir ) == false or _file.IsDir( dir ) == false then
		error_msg( "getFilesFromDir() was passed an invalid directory!" )
	end
	
	local files = {}
	local dirs = {}
	
	local result = _file.Find( dir .. "/*" )
	
	for i=1, table.getn( result ) do
		if result[ i ] == "." or result[ i ] == ".." then -- We just want to ignore these
		else
			if _file.IsDir( dir .. "/" .. result[ i ] ) == true then
				table.insert( dirs, result[ i ] )
			elseif _file.Exists( dir .. "/" .. result[ i ] ) == true then -- Double check it's a file
				table.insert( files, result[ i ] )
			end
		end
	end
	
	return files, dirs
end

function getFilesFromDirRecursive( dir )
	if _file.Exists( dir ) == false or _file.IsDir( dir ) == false then
		error_msg( "getFilesFromDirRecursive() was passed an invalid directory!" )
		return
	end
	
	local files = {}
	
	local result = _file.Find( dir .. "/*" )
	
	for i=1, table.getn( result ) do
		if result[ i ] == "." or result[ i ] == ".." then -- We just want to ignore these
		else
			if _file.IsDir( dir .. "/" .. result[ i ] ) == true then
				local temp = getFilesFromDirRecursive( dir .. "/" .. result[ i ] )
				for i=1, table.getn( temp ) do
					table.insert( files, temp[ i ] )
				end
			elseif _file.Exists( dir .. "/" .. result[ i ] ) == true then -- Double check it's a file
				table.insert( files, dir .. "/" .. result[ i ] )
			end
		end
	end
	
	return files
end

-- Strips a file's extension
function stripExt( file )
	return string.gsub( file, "^(.*)%..-$", "%1" )
end

-- This comes from lua.org, thanks for teaching millions lua, and providing this awesome code!
List = {}
function List.new ()
	return {first = 0, last = -1}
end


function List.pushleft (list, value)
	local first = list.first - 1
	list.first = first
	list[first] = value
end
    
function List.pushright (list, value)
	local last = list.last + 1
	list.last = last
	list[last] = value
end
    
function List.popleft (list)
	local first = list.first
	if first > list.last then error("list is empty") end
	local value = list[first]
	list[first] = nil        -- to allow garbage collection
	list.first = first + 1
	return value
end
    
function List.popright (list)
	local last = list.last
	if list.first > last then error("list is empty") end
	local value = list[last]
	list[last] = nil         -- to allow garbage collection
	list.last = last - 1
	return value
end
